-- pdnslog("pdns-recursor Lua script starting!", pdns.loglevels.Warning)

cjson = require "cjson"

function script_path()
   local str = debug.getinfo(2, "S").source:sub(2)
   return str:match("(.*/)")
end

local fw_servers = {}
local results = {}
local src_groups = {}
local rules = {}

math.randomseed(os.time())

local server_config = {}
local sysroot = script_path()
if sysroot == nil then
    sysroot = '/etc/powerdns'
end

function split(pString, pPattern)
   local Table = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pPattern
   local last_end = 1
   local s, e, cap = pString:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
     table.insert(Table,cap)
      end
      last_end = e+1
      s, e, cap = pString:find(fpat, last_end)
   end
   if last_end <= #pString then
      cap = pString:sub(last_end)
      table.insert(Table, cap)
   end
   return Table
end

function loadConfig()
    local f = assert(io.open(sysroot..'/pdns-recursor.json', 'r'))
    local jsonData = f:read("*all")
    io.close(f)
    jsonData = cjson.decode(jsonData)
    for k,v in pairs(jsonData.source_group) do
        if (type(src_groups[k])=="nil") then src_groups[k] = newNMG(); end

        for _k, ip in pairs(v) do
            -- print ('init mask '..ip)
            src_groups[k]:addMask(ip)
        end
    end

    for k,v in pairs(jsonData.result_group) do
        if (type(results[k])=="nil") then results[k] = {}; end

        for _k, rc in pairs(v) do
            table.insert(results[k], rc)
        end
    end

    for k,v in pairs(jsonData.forwarders) do
        fw_servers[k] = newCA(v)
    end

    rules = jsonData.rules
end

loadConfig()

function preresolve(dq)
    local queryName = dq.qname:toString()

    local queryNameList = split(queryName, "%.")
    local queryPartNum = table.getn(queryNameList)

    local i

    local parttens = {}

    local newQueryPartten = "."

    table.insert(parttens, newQueryPartten)
    for i = queryPartNum, 1, -1 do
        newQueryPartten = "." .. queryNameList[i] .. newQueryPartten
        table.insert(parttens, newQueryPartten)
    end
    table.insert(parttens, queryName)

    for i = queryPartNum + 2, 1, -1 do
        if type(rules[parttens[i]]) ~= "nil" then
            local ruleNum = table.getn(rules[parttens[i]])
            for ruleId = 1, ruleNum do
                local rule = rules[parttens[i]][ruleId]

                local has_matched = true
                if rule.src_group_id ~= nil then
                    -- print ("rule has source match required")
                    has_matched = src_groups[rule.src_group_id]:match(dq.localaddr)
                    dq.variable = true
                end
                if has_matched then
                    if rule.action == "result" then
                        -- print ("get result directly "..rule.rc_id)
                        local rc = results[rule.rc_id]
                        print (rc, table.getn(rc))
                        for rc_i = 1, table.getn(rc) do
                            dq:addAnswer(pdns[rc[rc_i].type], rc[rc_i].value, rc[rc_i].ttl)
                        end
                        return true;
                    end
                    -- print ("matched rule "..parttens[i].."   action = "..rule.action)
                    return false
                end
            end
        end
    end

    return false
end


function postgetns(dq)
    local queryName = dq.qname:toString()

    local queryNameList = split(queryName, "%.")
    local queryPartNum = table.getn(queryNameList)

    local i

    local parttens = {}

    local newQueryPartten = "."

    local ns_hasReset = false

    table.insert(parttens, newQueryPartten)
    for i = queryPartNum, 1, -1 do
        newQueryPartten = "." .. queryNameList[i] .. newQueryPartten
        table.insert(parttens, newQueryPartten)
    end
    table.insert(parttens, queryName)

    local inserted_ns = {}

    for i = queryPartNum + 2, 1, -1 do
        if type(rules[parttens[i]]) ~= "nil" then
            local ruleNum = table.getn(rules[parttens[i]])
            for ruleId = 1, ruleNum do
                local rule = rules[parttens[i]][ruleId]

                local has_matched = true
                if rule.src_group_id ~= nil then
                    -- print ("rule has source match required")
                    has_matched = src_groups[rule.src_group_id]:match(dq.localaddr)
                end
                if has_matched then
                    if rule.action == "forwarder" then
                        pdnslog("set "..parttens[i].." forward server from "..dq.remoteaddr:toString().." to "..fw_servers[rule.forwarder_id]:toString(), pdns.loglevels.Notice)
                        if ns_hasReset == false then
                            dq:resetNS()
                            ns_hasReset = true
                        end
                        if not inserted_ns[rule.forwarder_id] then
                            inserted_ns[rule.forwarder_id] = true
                            dq:addNS(fw_servers[rule.forwarder_id])
                        end

                        -- print ("set forward server to "..fw_servers[rule.forwarder_id]:toString())
                    end
                    -- print ("matched rule "..parttens[i].."   action = "..rule.action)
                end
            end
            if ns_hasReset then 
                return true
            end
        end
    end

    return true
end

